<?php declare(strict_types=1);

namespace Shopware\Core\Migration\Test;

use Doctrine\DBAL\Connection;
use PHPUnit\Framework\TestCase;
use Shopware\Core\Framework\Test\TestCaseBase\KernelTestBehaviour;
use Shopware\Core\Migration\Migration1595492052SeoUrl;
use Shopware\Core\Migration\V6_4\Migration1643366069AddSeoUrlUpdaterIndex;

class Migration1595492052SeoUrlTest extends TestCase
{
    use KernelTestBehaviour;

    public function testNoChanges(): void
    {
        $conn = $this->getContainer()->get(Connection::class);
        $expectedSchema = $conn->fetchAssoc('SHOW CREATE TABLE `seo_url`')['Create Table'];

        $migration = new Migration1595492052SeoUrl();

        $migration->update($conn);
        $actualSchema = $conn->fetchAssoc('SHOW CREATE TABLE `seo_url`')['Create Table'];
        static::assertSame($expectedSchema, $actualSchema, 'Schema changed!. Run init again to have clean state');

        $migration->updateDestructive($conn);
        $actualSchema = $conn->fetchAssoc('SHOW CREATE TABLE `seo_url`')['Create Table'];
        static::assertSame($expectedSchema, $actualSchema, 'Schema changed!. Run init again to have clean state');
    }

    public function testNoSeoUrlTable(): void
    {
        $conn = $this->getContainer()->get(Connection::class);
        $conn->executeUpdate('DROP TABLE `seo_url`');

        $migration = new Migration1595492052SeoUrl();
        $migration->update($conn);

        $m = new Migration1643366069AddSeoUrlUpdaterIndex();
        $m->update($conn);
        $exists = $conn->fetchColumn('SELECT COUNT(*) FROM `seo_url`') !== false;

        static::assertTrue($exists);
    }
}
