/// <reference types="Cypress" />

describe('Sales Channel: create product, change currency and language', () => {
    beforeEach(() => {
        cy.loginViaApi().then(() => {
            cy.createProductFixture();
        }).then(() => {
            cy.openInitialPage(Cypress.env('admin'));
        });
    });

    it('@package: create sales channel', () => {
        cy.intercept({
            url: `**/${Cypress.env('apiPath')}/search/sales-channel-domain`,
            method: 'POST'
        }).as('verifyDomain');
        cy.intercept({
            url: `**/${Cypress.env('apiPath')}/product-visibility`,
            method: 'POST'
        }).as('editProduct');
        cy.intercept({
            url: `**/${Cypress.env('apiPath')}/search/sales-channel`,
            method: 'POST'
        }).as('editSalesChannel');

        // Configure sales channel
        cy.goToSalesChannelDetail('E2E install test')
            .selectCountryForSalesChannel('Germany')
            .selectLanguageForSalesChannel('Deutsch')
            .selectCurrencyForSalesChannel('US-Dollar');

        // Add NL domain
        cy.get('.sw-card.sw-card--grid.sw-sales-channel-detail-domains').scrollIntoView();
        cy.get('.sw-sales-channel-detail__button-domain-add').contains('Domein toevoegen').click();
        cy.get('.sw-modal__title').contains('Nieuw domain maken');
        cy.get('.sw-field__url-input__prefix').click();
        cy.get('#sw-field--currentDomain-url').type('localhost:8000/nl');
        cy.get('.sw-sales-channel-detail-domains__domain-language-select').find('.sw-single-select__selection').click();
        cy.contains('.sw-select-result', 'Dutch').click();
        cy.get('.sw-sales-channel-detail-domains__domain-currency-select').find('.sw-single-select__selection').click();
        cy.contains('.sw-select-result', 'Euro').click();
        cy.contains('.sw-entity-single-select', 'Tekstfragment').find('.sw-entity-single-select__selection').click();
        cy.contains('.sw-select-result', 'LanguagePack nl-NL').click();
        cy.contains('.sw-button--primary', 'Domein toevoegen').click();
        cy.wait('@verifyDomain').its('response.statusCode').should('equal', 200);
        cy.get('.sw-sales-channel-detail-domains .sw-data-grid__body')
            .find('.sw-data-grid__row').should('have.length', 3);

        // Add DE domain
        cy.get('.sw-card.sw-card--grid.sw-sales-channel-detail-domains').scrollIntoView();
        cy.get('.sw-sales-channel-detail__button-domain-add').contains('Domein toevoegen').click();
        cy.get('.sw-modal__title').contains('Nieuw domain maken');
        cy.get('.sw-field__url-input__prefix').click();
        cy.get('#sw-field--currentDomain-url').type('localhost:8000/de');
        cy.get('.sw-sales-channel-detail-domains__domain-language-select').find('.sw-single-select__selection').click();
        cy.contains('.sw-select-result', 'Deutsch').click();
        cy.get('.sw-sales-channel-detail-domains__domain-currency-select').find('.sw-single-select__selection').click();
        cy.contains('.sw-select-result', 'Euro').click();
        cy.contains('.sw-entity-single-select', 'Tekstfragment').find('.sw-entity-single-select__selection').click();
        cy.contains('.sw-select-result', 'BASE de-DE').click();
        cy.contains('.sw-button--primary', 'Domein toevoegen').click();
        cy.wait('@verifyDomain').its('response.statusCode').should('equal', 200);
        cy.get('.sw-sales-channel-detail-domains .sw-data-grid__body')
            .find('.sw-data-grid__row').should('have.length', 4);

        // Add product
        cy.get('[title="Producten"]').scrollIntoView().click();
        cy.get('.sw-button.sw-button--ghost').click();
        cy.get('.sw-data-grid__body .sw-data-grid__cell--selection .sw-data-grid__cell-content').click();
        cy.get('.sw-data-grid__bulk-selected-label').should('include.text', 'Geselecteerd');
        cy.get('.sw-button.sw-button--primary.sw-button--small').click();
        cy.wait('@editProduct').its('response.statusCode').should('equal', 204);

        // Save sales channel
        cy.get('.sw-button-process.sw-sales-channel-detail__save-action').click();
        cy.wait('@editSalesChannel').its('response.statusCode').should('equal', 200);

        // Storefront
        cy.visit('/nl');
        cy.get('.header-search-input').type('Product name');
        cy.contains('.search-suggest-product-name', 'Product name').click();
        cy.get('.product-detail-price').contains('€ 49,98*')
        cy.get('.product-detail-buy .btn-buy').contains('In het winkelmandje').click();

        // Off canvas
        cy.get('.offcanvas.is-open').should('be.visible');
        cy.get('.cart-item-price').contains('€ 49,98*').should('be.visible');
        cy.get('a[title="Naar de kassa"]').should('be.visible');
        cy.contains('Doorgaan met winkelen').click();

        // Change currency
        cy.get('#currenciesDropdown-top-bar').click();
        cy.get('.dropdown-menu-right.show.top-bar-list').contains('$ USD').click();
        cy.get('#currenciesDropdown-top-bar').contains('$ US-Dollar');

        // Change language
        cy.get('#languagesDropdown-top-bar').click();
        cy.get('.dropdown-menu-right.show.top-bar-list').contains('Deutsch').click();
        cy.url().should('include', '/de');
        cy.get('#languagesDropdown-top-bar').contains('Deutsch');

        // Verify currency and language on detail page
        cy.get('.product-detail-price').contains('58,52 $*')
        cy.get('.product-detail-buy .btn-buy').contains('In den Warenkorb').should('be.visible');

        // Verify currency and language on canvas
        cy.get('.header-cart-icon').click({ force: true });
        cy.get('.offcanvas.is-open').should('be.visible');
        cy.get('.cart-item-price').contains('58,52 $*').should('be.visible');
        cy.get('a[title="Weiter zur Kasse"]').should('be.visible');
    });
});
