describe('Google Analytics: New analytics cookie in Cookie Consent Manager', () => {
    it('@cookies: There is a new analytics cookie in the manager if it is activated for the saleschannel', () => {
        cy.setAnalyticsFixtureToSalesChannel(true);
        cy.visit('/');

        cy.window().then((win) => {
            win.PluginManager.getPluginInstances('CookieConfiguration')[0].openOffCanvas();
            cy.get('.offcanvas-cookie').should('be.visible').then(() => {
                cy.get('.offcanvas-cookie').find('.offcanvas-cookie-group').should('have.length', 2);
                cy.get('.offcanvas-cookie-group').eq(1).contains('Statistic');

                cy.get('.offcanvas-cookie-group').eq(1).find('.offcanvas-cookie-entry').contains('Google Analytics');
            });
        });
        cy.setAnalyticsFixtureToSalesChannel(false);
    });

    it('@cookies: There is no statistics group in the cookie manager if google analytics is not activated for the saleschannel', () => {
        cy.setAnalyticsFixtureToSalesChannel(false);
        cy.visit('/');

        cy.window().then((win) => {
            win.PluginManager.getPluginInstances('CookieConfiguration')[0].openOffCanvas();
            cy.get('.offcanvas-cookie').should('be.visible').then(() => {
                cy.get('.offcanvas-cookie').find('.offcanvas-cookie-group').should('have.length', 1);
                cy.get('.offcanvas-cookie-group').should('not.contain', 'Statistic');
            });
        });
    });

    it('@cookies: Google Analytics cookies will be set and removed again', () => {
        cy.setAnalyticsFixtureToSalesChannel(true);
        cy.intercept({
            method: 'GET',
            url: '/cookie/offcanvas',
        }).as('cookieOffcanvas');

        cy.visit('/');

        cy.getCookie('_swag_ga_ga').should('be.null');

        cy.get('.js-cookie-configuration-button > .btn').click();
        cy.wait('@cookieOffcanvas')
            .its('response.statusCode').should('equal', 200);

        // waiting on the request is not sufficient on all systems, so we wait a second
        cy.wait(1000);

        cy.get('.offcanvas-cookie').should('be.visible');
        cy.get('.offcanvas-cookie-group').eq(1).find('.custom-control-label').first().click();
        cy.get('.js-offcanvas-cookie-submit').click();

        cy.reload();

        cy.window().then((win) => {
            cy.waitUntil(() => cy.getCookie('_swag_ga_ga').then(cookie => cookie && cookie.value !== null));

            cy.get('.offcanvas-cookie').should('not.be.visible').then(() => {
                win.PluginManager.getPluginInstances('CookieConfiguration')[0].openOffCanvas();
                cy.get('.offcanvas-cookie').should('be.visible').then(() => {
                    cy.get('.offcanvas-cookie-group').eq(1).find('.custom-control-label').first().click();
                    cy.get('.js-offcanvas-cookie-submit').click();

                    cy.waitUntil(() => cy.getCookie('_swag_ga_ga').then(cookie => !cookie || cookie.value === null));
                });
            });

            win.PluginManager.getPluginInstances('CookieConfiguration')[0].openOffCanvas();
        });
        cy.setAnalyticsFixtureToSalesChannel(false);
    });
});
